@@ -20,15 +20,18 @@ use AutoLoader;
 use Carp;
 @ISA       = qw(Set::Infinite);
 @EXPORT    = qw();
-@EXPORT_OK = qw(type $inf inf);
-$VERSION = (qw'$Revision: 1.24_17 $')[1];
+@EXPORT_OK = qw( $inf inf );
+$VERSION = (qw'$Revision: 1.25_03 $')[1];
 
 
 #----- initialize package globals
 
 $DEBUG                = 0;
 $Set::Infinite::TRACE = 0;
-Set::Infinite::type('Date::Set::ICal');
+
+my $Type;
+Date::Set->type('Date::Set::ICal');
+
 $inf = 10**10**10;
 sub inf { $inf }
 
@@ -55,13 +58,34 @@ $NEVER   = __PACKAGE__->new();
 
 #----- end: initialize package globals
 
+# override type() to use our default value --> $Type
+
+sub type {
+    # warn "Date::Set type: $_[-1]";
+    my $self = shift;
+    unless (@_) {
+        return ref($self) ? $self->{type} : $Type;
+    }
+    my $tmp_type = shift;
+    eval "use " . $tmp_type;
+    carp "Warning: can't start $tmp_type : $@" if $@;
+    if (ref($self))  {
+        $self->{type} = $tmp_type;
+        return $self;
+    }
+    else {
+        $Type = $tmp_type;
+        return $Type;
+    }
+}
+
 #----- our special version of new()
 # propagates dtstart 
 
 sub new {
     my $class = shift;
-    my $self = Set::Infinite->new(@_);
-    bless $self, __PACKAGE__;
+    my $self = $class->SUPER::new(@_);
+    bless $self, ( ref($class) || $class );
     # print " new ";
     $self->{dtstart} = ref($class) ? $class->{dtstart} : undef;
     $self->{wkst} =    ref($class) ? $class->{wkst}    : $WKST;
@@ -862,11 +886,12 @@ sub dtstart {    # start
     my ( $self, %parm ) = @_;
     unless (ref($self)) {
         # constructor
-        $self = $self->new( $parm{start}, $FUTURE );
+        $self = $self->new( $parm{start} )->offset( value => [ 0, $inf ] );   # $parm{start}, $FUTURE );
         $self->{dtstart} = $parm{start};
+        $self->trace( title => 'dtstart-new' );
         return $self;
     }
-    $self->_print( title => 'dtstart ' . join ( ':', %parm ) ) if $DEBUG;
+    $self->trace( title => 'dtstart ' . join ( ':', %parm ) );
     my $dt = $self->copy;
     $dt->{dtstart} = $parm{start};
     # print " dtstart $self->{dtstart}\n";
@@ -1053,7 +1078,7 @@ sub recur_by_rule {
         }
         $last_p = $p;
     }
-    print "\nRECUR_BY_RULE ",join(":",@_),"\n" if $DEBUG;
+    $self->trace( title => "recur_by_rule" );   # ".join(":",@_) );
 
     my %parm = (
         # FREQ     => 'YEARLY',  # whatever
@@ -1227,6 +1252,9 @@ sub recur_by_rule {
             #    # warn "failed: 'first' $size";
             # }
         }
+        else {
+            $b->{first} = [ $b->new(-$inf), $b->copy ];
+        }
 
 
         $b->trace_close( arg => $b );
@@ -1555,6 +1583,7 @@ sub _rrule_by {
             # reuse "base" since we already know it from FREQ
             # -- iterate through parameters
             $indexed = $NEVER;
+            # $DEBUG = 1;
             foreach (@indexed_byday) {
                 # parse parameters
                 my ( $index, $day ) = /([\-\+]?\d+)(\w\w)/;
@@ -1562,7 +1591,8 @@ sub _rrule_by {
                 $index-- if $index > 0;   # perl index starts in 0 instead of 1
                 $day = $WEEKDAY{$day};
                 # print " [Indexed BYDAY: $index $day ]\n" if $DEBUG;
-                my $weekday = $base->offset( 
+                ## my $weekday = $base->offset( 
+                my $weekday = $BYDAY->offset(
                     mode => 'offset', 
                     unit => 'weekdays', 
                     value => [ $day, $day ] );
@@ -1588,6 +1618,7 @@ sub _rrule_by {
 
         $has{months} = 1;
         $has{days}   = 1;
+        # $DEBUG = 0;
     }    # end: BYDAY
     # }}}
 
@@ -1876,6 +1907,21 @@ sub DESTROY {}
 # currently defined are:
 #   next_XXX  (next_day, next_month ...), this_XXX,  prev_XXX
 
+# This is experimental code, originally developed for DateTime::Set:
+#
+# If I can't do something, I check if the first 'leaf' can do it.
+# For example:
+#   $set_10 = $set->add( seconds => 10 );
+# is a shortcut to:
+#   $set_10 = $set->new( $set->min->add( seconds => 10 ) );
+#
+
+my %Is_Leaf_Subroutine = (
+    add => 1,
+    # clone() is a function
+    # ical() is a function
+);
+
 sub AUTOLOAD {
 
     no strict 'refs';
@@ -1918,6 +1964,40 @@ sub AUTOLOAD {
             goto &$sub;
 
         }
+        # ok, so we fell through.
+        # let's see if this is a Date::ICal method
+
+        # my $sub = $1;
+        my $self = shift;
+        my $leaf = $self->min;  # get first leaf
+
+        # get over the memoization
+        $leaf = $leaf->date_ical if UNIVERSAL::can( $leaf, 'date_ical');
+        # warn "leaf is a '". ref( $leaf ) . "'";
+
+        # warn "leaf value is ". $leaf->ical ." is a '". ref( $leaf ). "' sub is '". $sub. "' param @_";
+        if ( UNIVERSAL::can( $leaf, $sub ) ) {
+            # we have different calling modes in leaf class - that's bad.
+            if (exists $Is_Leaf_Subroutine{$sub} ) {
+                # calling mode is 'subroutine'
+                $leaf = $leaf->clone;
+                $leaf->$sub(@_);
+                # warn "sub result is ". $leaf->ical ." ";
+            }
+            else {
+                # calling mode is 'function'
+                $leaf = $leaf->$sub(@_);
+                # warn "function result is ". $leaf;
+            }
+            # warn "result is a '". ref( $leaf ) . "'";
+            $leaf = $self->new($leaf) if ref($leaf) eq 'Date::ICal';
+            return $leaf;
+        }
+
+        # end Date::ICal interface
+
+        Carp::croak( __PACKAGE__ . $AUTOLOAD . " is malformed in AUTOLOAD" );
+
     } else {
         Carp::croak( __PACKAGE__ . $AUTOLOAD . " is malformed in AUTOLOAD" );
     }
